## get_osc.py
#
#  Get OSC data and decode
#
#############################################################################

import GameLogic as gl
import socket
from OSC3.OSC import decodeOSC
from time import sleep

scenes = gl.getSceneList()

def main():
    ip = gl.ip_multicast
    buffer_size = 1024
    port = gl.port_in

    connect_blender(ip, port, buffer_size)
    get_OSC_data(buffer_size)

    # Calcul et affichage du ratio de réception, le socket ne récupère pas toujours des datas à chaque frame
    gl.frame_nbr += 1
    if gl.frame_nbr == 1000:
        gl.frame_nbr, gl.socket_failed = 0, 0
    if gl.frame_nbr != 0: gl.socket_ratio = (gl.frame_nbr - gl.socket_failed)/gl.frame_nbr

def connect_blender(ip, port, buffer_size):
    # Connexion de Blender en Multicast
    if not gl.connected :
        try:

            gl.sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
            gl.sock.setblocking(0)
            gl.sock.settimeout(0.010)
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            # Taille du buffer UDP pour le vider entièrement à chaque lecture
            # et ne pas avoir de latence
            gl.sock.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, buffer_size)
            gl.sock.bind((ip, port))

            print('Get OSC on IP={} Port={} Buffer={}'.format(ip, port, buffer_size))
            gl.connected = True

        except:
            print('No connected to IP={} Port={}'.format(ip, port),
                                        ", gl.connected =", gl.connected)
            pass

def get_OSC_data(buffer_size):
    # Si blender est connecté, récupération des messages OSC
    if gl.connected :
        try:
            raw_data = gl.sock.recv(buffer_size)
            data = decodeOSC(raw_data)
            #print("raw_data in get_osc.py =", raw_data)
            if gl.tempo_globale == 0:
                print("data in get_osc.py =", data)
            #print("OSC message received in get_osc.py")
            recup_messages(data)
        except socket.error:
            gl.socket_failed += 1
            pass

def recup_messages(data):
    ''' Pour 1 joueur
     data =
    [
    '#bundle', 0.0, ['/for_all_players', ',ssffsffsissssi', '192.168.1.4', '/ball', 8, 0, '/bat', 2, 1,
                                                               '/score', 9, '/name', 'ds', 'classement', 'ds', 0],
    ['/final_display', ',s', ''],
    ['/ball_reset', ',i', 0],
    ['/general_reset', ',i', 0]
    ]
    '''
    # Cut '#bundle', 0.0
    data = data[2:]
    # récup des messages
    for item in data:
        if item[0] == '/for_all_players':
            # Cette fonction a été créèe après le script players_data.py qui utilise gl.data
            # pour mettre en forme gl.players_data
            gl.data = data[0]
        elif item[0] == '/final_display':
            # item = ['/final_display', ',s', 'j2']
            if item[2] != 0:
                final_display(winner=item[2])
            else:
                gl.rank_scene = 0
                gl.text_rank = 0
        elif item[0] == '/reset_ball_svr':
            if item[2] == 1:
                ball_reset()
        elif item[0] == '/general_reset_svr':
            if item[2] == 1:
                general_reset()
        elif item[0] == '/quit_svr':
            if item[2] != "":
               quit_game(item[2])
        elif item[0] == '/score_reset':
            if item[2] == 1:
                reset_score()

def final_display(winner):
    # Lancé si le message '/final_display' = winner et non False
    print("OSC message /final_display received")
    # Affichage de la scène Rank
    if gl.level > 1:
        # Si Rank n'est pas encore affiché
        if gl.rank_scene != 2:
            # Provoque l'affichage de la scène Rank
            gl.rank_scene = 1
    # Affichage en bas d'écran de ce qui se passe
    gl.text_rank = 1

def ball_reset():
    # Lancé si le message '/ball_reset' = 1
    print("OSC message /ball_reset received: balle au centre")
    gl.ball.localPosition = [3, 3, 1]

def general_reset():
    # Lancé si le message '/general_reset' = 1
    print("OSC message /general_reset received: new_game(), reset_variables(), reset_score()")
    new_game()

def quit_game(joueur):
    # Reçu si un joueur quitte
    print("Un joueur a quitté le jeu:", joueur)

def reset_score():
    # Reset des scores du niveau en cours
    print ("OSC message /score_reset received: Je reset les scores")
    for i in range(gl.level):
        gl.goal[i]["score"] = 9

def reset_variables():
    # Reset de variables pour repartir à zéro
    reset_score()
    gl.rank_display_bug = 0
    gl.tempo_globale = 0

    # level 1 only
    gl.level1_rated = 0 # si classement du level 1 fait dans game.py
    gl.classement_level1 = {}
    gl.tempo_rank_level1 = 0

    # pour afficher le classement si j'ai perdu
    gl.rank_scene = 0

    # Le dictionnaire de classement des joueurs
    gl.classement = {}

    # blocage de la bat si j'ai fini
    gl.block = 0

def new_game():
    reset_variables()
    ### Supprime la scène précédente, ajoute la scene 1
    for scn in scenes :
        if "Rank" in scn.name:
            scn.end()
        if "_players" in scn.name:
            scn.end()

cont = gl.getCurrentController()
sens = cont.sensors["Get OSC"]
if sens.positive:
    main()
