#!/usr/bin/env pypy
# -*- coding: UTF-8 -*-

#!/usr/bin/env python
#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################

'''
To use this script with another project, change handler for exemple:
    To receive all message, without sub-messsage:
        self.receiver.addCallback("/*", some_handler)
    or, with touchosc, for example:
        self.receiver.addCallback("/1/*", some_handler)
    and create some_handler
'''

import os
import sys
import threading
from time import time, sleep

from txosc import async, dispatch, osc
from twisted.internet import reactor

import managementDict


class UDPMulticastSender(object):
    def __init__(self, port, host):
        self.port = port
        self.host = host
        self.client = async.DatagramClientProtocol()
        self._client_port = reactor.listenUDP(0, self.client)
        print("Sending multicast on: %s : %s" % (self.host, self.port))

    def send_message(self, message):
        self.client.send(message, (self.host, self.port))


class UDPMulticastReceiver(object):
    def __init__(self, port, host):
        self.port = port
        self.host = host
        self.receiver = dispatch.Receiver()
        self._server_port = reactor.listenMulticast(self.port,
                                async.MulticastDatagramServerProtocol(self.receiver, self.host), listenMultiple=True)
        print("Listening multicast on: %s : %s" % (self.host, self.port))
        # handler
        self.receiver.addCallback("/player_data", ServerMain_objet.player_data_handler)
        self.receiver.addCallback("/reset_ball", ServerMain_objet.reset_ball_handler)
        self.receiver.addCallback("/reset_game", ServerMain_objet.reset_game_handler)
        self.receiver.addCallback("/quit", ServerMain_objet.quit_handler)

class ServerMain(object):
    def __init__(self, port_multicast_in, port_multicast_out,host_multicast):
        self.port_multicast_in = port_multicast_in
        self.port_multicast_out = port_multicast_out
        self.host_multicast = host_multicast

        # Dictionnaire de centralisation des messages et des joueurs
        self.gameDict = managementDict.GameManagement()
        self.winner = 0
        self.since = 0

        # Fréquence d' ACTA
        self.t_connected = time()

        # messages particuliers
        self.msg_reset_ball = osc.Message("/ball_reset")
        self.msg_reset_ball.add(0)
        self.msg_new = osc.Message("/general_reset")
        self.msg_new.add(0)
        self.msg_final = osc.Message("/final_display")
        self.msg_final.add(0)
        self.msg_quit_svr = osc.Message("/quit_svr")
        self.msg_quit_svr.add("")
        self.msg_score_reset = osc.Message("/score_reset")
        self.msg_score_reset.add(0)

    def game_server(self):
        # Instance de twisted avec txosc
        self.multicast_sender = UDPMulticastSender(self.port_multicast_out, self.host_multicast)
        self.multicast_reicever = UDPMulticastReceiver(self.port_multicast_in, self.host_multicast)
        print "Game Server du jeu MultiPass For Freedom\n Press q + Enter to quit"
        print "Je lance Reactor"
        reactor.run()

    def player_data_handler(self, message, address):
        '''
        Cette fonction est appelée à chaque réception de message.
        Ajout des datas à gameDict, gestion des déconnectés, gestion du gagnant, réenvoi de datas
        '''
        # Ajout à gameDict
        self.gameDict.insert_data_in_dict(message.getValues(), address)

        # Gestion des connectés
        self.ACTA()

        # The winner
        self.somebody_win()

    def ACTA(self):
        # Gestion des déconnectés toutes les secondes
        if time() - self.t_connected > 1:
            self.t_connected = time()
            connected = self.gameDict.delete_disconnected_player()
            #print "Les connectés sont:", connected

    def send2players(self):
        '''La fréquende d'envoi doit être un peu supérieure à 60 Hz, le FPS de Blender est au mieux de 60'''
        while True:
            sleep(0.015)
            # Création d'un bundle qui contiendra tous les messages
            b = self.create_bundle()
            # Envoi
            self.multicast_sender.send_message(b)

    def create_bundle(self):
        # Création d'un bundle qui contiendra tous les messages
        b = osc.Bundle()

        # Message du dictionnaire gameDict
        msg_for_all_players = self.gameDict.create_msg_for_all_players()
        b.add(msg_for_all_players)
        # Message du winner
        b.add(self.msg_final)

        # Message reset ball
        b.add(self.msg_reset_ball)
        # Remise à zéro
        self.msg_reset_ball = self.reinit_message("/reset_ball_svr", 0)

        # Message reset general
        b.add(self.msg_new)
        # Remise à zéro
        self.msg_new = self.reinit_message("/general_reset_svr", 0)

        # Message quit_svr
        b.add(self.msg_quit_svr)
        # Remise à zéro
        self.msg_quit_svr = self.reinit_message("/quit_svr", "")

        # Message score_reset
        b.add(self.msg_score_reset)
        # Remise à zéro
        self.msg_score_reset = self.reinit_message("/score_reset", 0)

        return b

    def reinit_message(self, msg, value):
        # Réinitiale un message après envoi
        msg_reinit = osc.Message(msg)
        msg_reinit.add(value)
        return msg_reinit

    def reset_ball_handler(self, message, address):
        print ("/reset_ball received")
        self.msg_reset_ball = osc.Message("/reset_ball_svr")
        self.msg_reset_ball.add(1)

    def reset_game_handler(self, message, address):
        print ("/reset_game received")
        # Send message /general_reset
        self.msg_new = osc.Message("/general_reset_svr")
        self.msg_new.add(1)

    def somebody_win(self):
        # Cette fonction est lancée à chaque réception, donc 120 Hz avec 2 joueurs
        self.winner = self.gameDict.who_is_the_winner()
        self.since = self.gameDict.winner_since()

        self.msg_final = self.reinit_message("/final_display", self.winner)

        if self.since > 1.5:
            # Remise à 9 de tous les scores de tous les joueurs
            # Ce message doit être envoyé plusieurs fois
            self.msg_score_reset = self.reinit_message("/score_reset", 1)
            ##print "Le gagnant est", self.winner, "depuis", self.since
            ##print "Je reset les scores"


    def quit_handler(self, message, address):
        # Only to display in blender terminal
        print "Quit received from:", address
        self.msg_quit_svr = osc.Message("/reset_ball_svr")
        self.msg_quit_svr.add(0)

def thread_keyboard():
    '''doc'''
    while True:
        touche = sys.stdin.read(1)
        if touche == 'q':
            print("Exit")
            os._exit(0)

if __name__ == "__main__":
    host_multicast = '224.0.0.14'
    port_multicast_out = 18888
    port_multicast_in  = 19999

    # Attente d'une saisie clavier
    thread1 = threading.Thread(target=thread_keyboard)
    thread1.start()

    # set instance of ServerMain class
    ServerMain_objet = ServerMain(port_multicast_in, port_multicast_out, host_multicast)

    # Envoi des bundles
    thread2 = threading.Thread(target=ServerMain_objet.send2players)
    thread2.start()
    print "L'envoi des bundles commence"

    # Run server
    ServerMain_objet.game_server()
    # La fonction game_server() se termine par reactor.run() qui lance l'envoi et la réception de twisted,
    # l'exécution de cette fonction s'arrête le script ici.
    print "Je ne passe jamais ici"
